/*
 * $LastChangedBy$
 * $LastChangedDate$
 * $LastChangedRevision$
 * $HeadURL$
 *
 * Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
 */

import Foundation
import UIKit

@objc public class BlueConicClient: NSObject {

    // Static instance of the BlueConicClient
    internal struct BlueConicSharedInstances {
        static var instance: BlueConicClientImpl!
        static var interface: BlueConicClient!
    }

    private static var __once: () = {
        BlueConicSharedInstances.interface = BlueConicClient()
        BlueConicSharedInstances.instance = BlueConicClientImpl()
        Timer.scheduledTimer(timeInterval: 5.0, target: BlueConicSharedInstances.instance!,
                             selector: #selector(BlueConicClientImpl.sync), userInfo: nil, repeats: true)
    }()

    /**
     Get an instance of the BlueConic client.
     &lt;pre&gt:
     // Swift:
     let client: BlueConicClient = BlueConicClient.getInstance(self)

     // Objective-C:
     BlueConicClient* client = [BlueConicClient getInstance:self];
     &lt;/pre&gt:

     - parameter context: The application context.

     - returns: The BlueConic client instance.
     */
    @objc public class func getInstance(_ context: UIViewController?) -> BlueConicClient {
        _ = BlueConicClient.__once

        BlueConicSharedInstances.instance.setContext(context)
        return BlueConicSharedInstances.interface
    }

    override init() {
    }

    /**
     Returns the first value for a given profile property.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let hobby: String = client.getProfileValue(&quot;hobby&quot;)

     // Objective-C:
     NSString* hobby = [client getProfileValue:@&quot;hobby&quot;];
     &lt;/pre&gt:

     - parameter property: The profile property to get the values for.

     - returns: The first value
     */
    @objc public func getProfileValue(_ property: String) -> String {
        var result = ""
        BlueConicClient.synchronized({
            result = BlueConicSharedInstances.instance.getProfileValue(property)
        })
        return result
    }

    /**
     Return the values for a given profile property.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let hobbies: [String] = client.getProfileValues(&quot;hobbies&quot;)

     // Objective-C:
     NSArray* hobbies = [client getProfileValues:@&quot;hobbies&quot;];
     &lt;/pre&gt:

     - parameter property: The profile property to get the values for.

     - returns: A collection containing the values.
     */
    @objc public func getProfileValues(_ property: String) -> [String]? {
        var result: [String]?
        BlueConicClient.synchronized({
            result = BlueConicSharedInstances.instance.getProfileValues(property)
        })
        return result
    }

    /**
     Returns the current ViewController.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let viewController = client.getViewController()

     // Objective-C:
     UIViewController* viewController = [client getViewController];
     &lt;/pre&gt:

     - returns:     The current ViewController.
     */
    @objc public func getViewController() -> UIViewController? {
        var result: UIViewController?
        BlueConicClient.synchronized({
            result = BlueConicSharedInstances.instance.getViewController()
        })
        return result
    }

    /**
     Returns a view component based on the given identifier or <code>nil</code> is no match is found.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     @IBOutlet weak var view: UIView!
     let view: UIView? = client.getView(&quot;#view&quot;)

     // Objective-C:
     @property (weak, nonatomic) IBOutlet UIView* view;
     UIView* view = [client getView:@&quot;#view&quot;];
     &lt;/pre&gt:

     - parameter selector: The Identifier, e.g. &quot;#view&quot;.

     - returns: The view or <code>nil</code>
     */
    @objc public func getView(_ selector: String) -> UIView? {
        var result: UIView?
        BlueConicClient.synchronized({
            result = BlueConicSharedInstances.instance.getView(selector)
        })
        return result

    }

    /**
     Returns the screenName either set in createEvent or the ViewControllers title.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.createEvent(&quot;PAGEVIEW&quot;, properties: [&quot;screenName&quot;: &quot;Main/HOMETAB&quot;])
     var screenName: String = client.getScreenName()

     // Objective-C:
     [client createEvent:@&quot;PAGEVIEW&quot; properties:@{@&quot;screenName&quot;: @&quot;MAIN/HOMETAB&quot;}];
     NSString* screenName = [client getScreenName];
     &lt;/pre&gt:

     - returns: The screen name
     */
    @objc public func getScreenName() -> String {
        var result = ""
        BlueConicClient.synchronized({
            result = BlueConicSharedInstances.instance.getScreenName()
        })
        return result
    }

    /**
     Adds a single property value to the profile.
     If there are already values for a property the new value will be added.
     Values for a property need to be unique; passing the same value multiple times will have no effect.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.addProfileValue(&quot;hobbies&quot;, value:&quot;tennis&quot;)

     // Objective-C:
     [client addProfileValue:@&quot;hobbies&quot; value:@&quot;tennis&quot;];
     &lt;/pre&gt:

     - parameter property: The profile property to add the values for.
     - parameter value: The property value to add to the profile.
     */
    @objc public func addProfileValue(_ property: String, value: String) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.addProfileValue(property, value: value)
            }
        })
    }

    /**
     Adds property values to the profile. The values from the collection are added to the profile.
     If there are already values for a property the new values will be added.
     Values for a property need to be unique; passing the same values multiple times will have no effect.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let hobbyArray = [&quot;tennis&quot;, &quot;soccer&quot;]
     client.addProfileValues(&quot;hobbies&quot;, values:hobbyArray)

     // Objective-C:
     NSArray* hobbyArray = [NSArray arrayWithObjects:@&quot;tennis&quot;, @&quot;soccer&quot;, nil];
     [client addProfileValues:@&quot;hobbies&quot; values:hobbyArray];
     &lt;/pre&gt:

     - parameter property: The profile property to add the values for.
     - parameter values: The property values to add to the profile.
     */
    @objc public func addProfileValues(_ property: String, values: [String]) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.addProfileValues(property, values: values)
            }
        })
    }

    /**
     Sets values on the profile. Passing a property that was already set with values will cause for the old values to be removed.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.setProfileValue(&quot;hobbies&quot;, value:&quot;tennis&quot;)

     // Objective-C:
     [client setProfileValue:@&quot;hobbies&quot; value:@&quot;tennis&quot;];
     &lt;/pre&gt:

     - parameter name: The profile property to add the values for.
     - parameter value: The profile value to store.
     */
    @objc public func setProfileValue(_ name: String, value: String) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.setProfileValue(name, value: value)
            }
        })
    }
    
    
    /**
     Sets BlueConic Enabled. Passing a true of false to enable or disable BlueConic functionality.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.setEnabled(false)

     // Objective-C:
     BOOL isEnabled = [client isEnabled];
     &lt;/pre&gt:

     - returns name: The profile property to add the values for.
     - parameter value: The profile value to store.
     */
    @objc public func setEnabled(_ isEnabled: Bool) {
        BlueConicClient.synchronized({
            BlueConicSharedInstances.instance.setEnabled(isEnabled)
        })
    }
    
    /**
     Return the satus of BlueConic, whether it is enabled or disabled.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let isEnabled = client.isEnabled()

     // Objective-C:
     [client isEnabled];
     &lt;/pre&gt:

     - returns: isEnabled Whether the BlueConicClient is enabled or disabled.
     - parameter value: The profile value to store.
     */
    @objc public func isEnabled() -> Bool {
        var result = true
        BlueConicClient.synchronized({
            result = BlueConicSharedInstances.instance.isEnabled()
        })
        return result
    }

    /**
     Return the ID of the profile

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     var profileId: String? = client.getProfileId()

     // Objective-C:
     NSString* profileId = [client getProfileId];
     &lt;/pre&gt:

     - returns: The ID of the profile
     */
    @objc public func getProfileId() -> String? {
        var profileId: String?
        BlueConicClient.synchronized({
            profileId = BlueConicSharedInstances.instance.getProfileId()
        })
        return profileId

    }

    /**
     Sets values on the profile. Passing a property that was already set with values will cause for the old values to be removed.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let hobbyArray = [&quot;tennis&quot;, &quot;soccer&quot;]
     client.setProfileValues("hobbies", values:hobbyArray)

     // Objective-C:
     NSArray* hobbyArray = [NSArray arrayWithObjects:@&quot;tennis&quot;, @&quot;soccer&quot;, nil];
     [client setProfileValues:@&quot;hobbies&quot; values:hobbyArray];
     &lt;/pre&gt:

     - parameter name: The profile property to add the values for.
     - parameter values: The profile values to store.
     */
    @objc public func setProfileValues(_ name: String, values: [String]) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.setProfileValues(name, values: values)
            }
        })
    }

    /**
     Setter for the locale to get the parameters for. By default, the default locale configured in BlueConic is used.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.setLocale(&quot;en_US&quot;)

     // Objective-C:
     [client setLocale:@&quot;en_US&quot;];
     &lt;/pre&gt:

     - parameter locale: The locale, e.g. 'en_US'.
     */
    @objc public func setLocale(_ locale: String) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.setLocale(locale)
            }
        })
    }

    /**
     Checks whether the app was started with simulator data. If so we try to get the username and the the mobile
     session id to connect to the simulator. The intent should look like:
     &quot;&lt;appID&gt;://&lt;hostname&gt;/&lt;username&gt;/&lt;mobilesSessionId&gt;&quot;.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     // Implement in AppDelegate.swift
     func application(application: UIApplication, openURL url: NSURL, sourceApplication: String?, annotation: AnyObject) -> Bool {
     BlueConicClient.getInstance(nil).setURL(url)
     return true
     }


     // Objective-C:
     // Implement in AppDelegate.m
     - (BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation {
     [[BlueConicClient getInstance:nil] setURL:url];
     return YES;
     }
     &lt;/pre&gt:

     - parameter url: The url retrieved from application.
     */
    @objc public func setURL(_ url: URL) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.setURL(url)
            }
        })
    }
    
    /**
     Returns the privacy legislation of this profile

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let privacyLegislation: String = client.getPrivacyLegislation()

     // Objective-C:
     NSString* privacyLegislation = [client getPrivacyLegislation];
     &lt;/pre&gt:

     - returns: The privacy legislation of the profile
     */
    @objc public func getPrivacyLegislation() -> String? {
        var result: String?
        BlueConicClient.synchronized({
            result = BlueConicSharedInstances.instance.getProfileValue("privacy_legislation")
        })
        return result
    }
    
    /**
     Returns the consented objectives of this profile

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let consentedObjectives: String = client.getConsentedObjectives()

     // Objective-C:
     NSString* consentedObjectives = [client getConsentedObjectives];
     &lt;/pre&gt:

     - returns: The consented objectives of the profile
     */
    @objc public func getConsentedObjectives() -> [String]? {
        var result: [String]?
        BlueConicClient.synchronized({
            result = BlueConicSharedInstances.instance.getProfileValues("consented_objectives")
        })
        return result
    }
    
    /**
     Returns the refused objectives of this profile

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let refusedObjectives: String = client.getRefusedObjectives()

     // Objective-C:
     NSString* refusedObjectives = [client getRefusedObjectives];
     &lt;/pre&gt:

     - returns: The refused objectives of the profile
     */
    @objc public func getRefusedObjectives() -> [String]? {
        var result: [String]?
        BlueConicClient.synchronized({
            result = BlueConicSharedInstances.instance.getProfileValues("refused_objectives")
        })
        return result
    }
    
    /**
     Setter for the privacy legislation for this profile

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.setPrivacyLegislation()

     // Objective-C:
     [client setPrivacyLegislation:@&quot;GDPR&quot;];
     &lt;/pre&gt:

     - parameter legislation: The legislation, e.g. 'GDPR'.
     */
    @objc public func setPrivacyLegislation(_ legislation: String) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.setProfileValue("privacy_legislation", value: legislation)
            }
        })
    }
    
    /**
     Adds a consented objective to this profile.
     If there are already values for a property the new value will be added.
     Values for a property need to be unique; passing the same value multiple times will have no effect.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.addConsentedObjective(&quot;tracking&quot;)

     // Objective-C:
     [client addConsentedObjective:@&quot;tracking&quot;];
     &lt;/pre&gt:

     - parameter objectiveId: The objective ID to consent.
     */
    @objc public func addConsentedObjective(_ objectiveId: String) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.addProfileValue("consented_objectives", value: objectiveId)
            }
        })
    }
    
    /**
      Adds a refused objective to this profile.
      If there are already values for a property the new value will be added.
      Values for a property need to be unique; passing the same value multiple times will have no effect.

      &lt;h4&gt;Example&lt;/h4&gt;
      &lt;pre&gt:
      // Swift:
      client.addRefusedObjective(&quot;tracking&quot;)

      // Objective-C:
      [client addRefusedObjective:@&quot;tracking&quot;];
      &lt;/pre&gt:

      - parameter objectiveId: The objective ID to refuse.

      */
     @objc public func addRefusedObjective(_ objectiveId: String) {
         BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.addProfileValue("consented_objectives", value: objectiveId)
            }
         })
     }
    
    /**
     Sets consented objectives to this profile.
     Passing a property that was already set with values will cause for the old values to be removed.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let objectiveIds = [&quot;tracking&quot;, &quot;advertising&quot;]
     client.setConsentedObjectives(objectiveIds)

     // Objective-C:
     NSArray* objectiveIds = [NSArray arrayWithObjects:@&quot;tracking&quot;, @&quot;advertising&quot;, nil];
     [client setConsentedObjectives:objectiveIds];
     &lt;/pre&gt:

     - parameter objectiveIds: The consented objective IDs to set.
     */
    @objc public func setConsentedObjectives(_ objectiveIds: [String]) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.setProfileValues("consented_objectives", values: objectiveIds)
            }
        })
    }
    
    /**
     Sets refused objectives to this profile.
     Passing a property that was already set with values will cause for the old values to be removed.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     let objectiveIds = [&quot;tracking&quot;, &quot;advertising&quot;]
     client.setRefusedObjectives(objectiveIds)

     // Objective-C:
     NSArray* objectiveIds = [NSArray arrayWithObjects:@&quot;tracking&quot;, @&quot;advertising&quot;, nil];
     [client setRefusedObjectives:objectiveIds];
     &lt;/pre&gt:

     - parameter objectiveIds: The refused objective IDs to set.
     */
    @objc public func setRefusedObjectives(_ objectiveIds: [String]) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.setProfileValues("refused_objectives", values: objectiveIds)
            }
        })
    }
    
    /**
     Registers an event of the specified type with the given properties. For a &quot;PAGEVIEW&quot; event a screen name can be
     passed, so interactions can be restricted on the where tab in BlueConic.
     For a &quot;VIEW&quot;, &quot;CLICK&quot; or &quot;CONVERSION&quot; event an interactionId should be passed to register the event for.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.createEvent(&quot;CUSTOMEVENT&quot;, data: [&quot;context&quot;: self._context.getInteractionId()])

     // Objective-C:
     [client createEvent:@&quot;CUSTOMEVENT&quot; properties:@{@&quot;interactionId&quot;: [self._context getInteractionId]}];
     &lt;/pre&gt:
     
     
     // Swift:
     let contextArray = [&quot;tennis&quot;, &quot;soccer&quot;]
     client.createEvent(&quot;CUSTOMEVENT&quot;, data:contextArray)

     // Objective-C:
     NSArray* contextArray = [NSArray arrayWithObjects:@&quot;tennis&quot;, @&quot;soccer&quot;, nil];
     [client createEvent:@&quot;CUSTOMEVENT&quot; data:contextArray];

     - parameter eventType: The event type. (e.g: &quot;PAGEVIEW&quot;, &quot;VIEW&quot;, &quot;CLICK&quot;, &quot;CONVERSION&quot;)
     - parameter data: A map with data for the event.

     */
    @objc public func createEvent(_ eventType: String, data: [String: Any]?) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.createEvent(eventType, data: data)
            }
        })
    }
    
    
    /**
     Registers an event of the specified type with the given properties. For a &quot;PAGEVIEW&quot; event a screen name can be
     passed, so interactions can be restricted on the where tab in BlueConic.
     For a &quot;VIEW&quot;, &quot;CLICK&quot; or &quot;CONVERSION&quot; event an interactionId should be passed to register the event for.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.createEvent(&quot;PAGEVIEW&quot;, properties: [&quot;screenName&quot;: &quot;Main/HOMETAB&quot;])
     client.createEvent(&quot;CLICK&quot;, properties: [&quot;interactionId&quot;: self._context.getInteractionId()])

     // Objective-C:
     [client createEvent:@&quot;PAGEVIEW&quot; properties:@{@&quot;screenName&quot;: @&quot;MAIN/HOMETAB&quot;}];
     [client createEvent:@&quot;CLICK&quot; properties:@{@&quot;interactionId&quot;: [self._context getInteractionId]}];
     &lt;/pre&gt:

     - parameter eventType: The event type. (e.g: &quot;PAGEVIEW&quot;, &quot;VIEW&quot;, &quot;CLICK&quot;, &quot;CONVERSION&quot;)
     - parameter properties: A map with properties for the event.

     */
    @objc public func createEvent(_ eventType: String, properties: [String: String]?) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.createEvent(eventType, properties: properties)
            }
        })
    }
    
    /**
     Register plugin class. Assign a custom plugin to the list of available plugins.
     Which are then validated during the PAGEVIEW event, which creates and executes the correct plugin for you.

     &lt;h4&gt;Example&lt;/h4&gt;
     &lt;pre&gt:
     // Swift:
     client.registerPluginClass(TestPlugin.self)

     // Objective-C:
     [client registerPluginClass: [TestPlugin class]];
     &lt;/pre&gt:

     - parameter pluginClass: The  class of the plugin.
     */
    @objc public func registerPluginClass(_ pluginClass: AnyClass) {
        BlueConicClient.synchronized({
            if BlueConicSharedInstances.instance.isEnabled() {
                BlueConicSharedInstances.instance.registerPluginClass(pluginClass)
            }
        })
    }

    /**
     Private function.
     
     Returns all available connections.
     This function is used by Interaction Context, to get a connection based on the id.
     
     - returns: All connections from the last pageview
     */
    @objc internal func getConnections() -> [Connection] {
        var connections = [Connection]()
        BlueConicClient.synchronized({
            connections = BlueConicSharedInstances.instance.getConnections()
        })
        return connections
    }
}
