/*
* $LastChangedBy$
* $LastChangedDate$
* $LastChangedRevision$
* $HeadURL$
*
* Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
*/
import Foundation
import UIKit

/**
Interface InteractionContext
*/
@objc public class InteractionContext: NSObject {
    private var _id: String?
    private var _name: String?
    private var _type: String?
    
    private var _positionIdentifier: String?
    private var _positionName: String?
    
    private var _parameters = [String: [String]]()
    private var _context: UIViewController?
    private var _locale: String?
    private var _defaultLocale: String?
    
    private var _pluginType: String?
    
    private var _dialogueId: String?
    private var _dialogueName: String?

    /**
    Default init() for BlueConicCommitLog

    - parameter interaction: is a collection of data that is retrieved from the server
    - parameter context: The current ViewController
    - parameter locale: The locale the interaction should use, e.g. 'en_US'
    */
    @objc init(interaction: [String: Any], context: UIViewController?, locale: String?) {
        super.init()
        self._id = interaction[Constants.Interactions.ID] as? String
        self._name = interaction[Constants.Interactions.NAME] as? String
        self._type = interaction[Constants.Interactions.TYPE] as? String
        self._pluginType = interaction[Constants.Interactions.PLUGIN_TYPE] as? String
        
        self._positionIdentifier = interaction[Constants.Interactions.POSITION_KEY] as? String
        self._positionName = interaction[Constants.Interactions.POSITION_NAME] as? String
        self._defaultLocale = interaction[Constants.Interactions.DEFAULT_LOCALE] as? String
        
        // Dialogue related properties
        self._dialogueId = interaction[Constants.Interactions.DIALOGUE_ID] as? String
        self._dialogueName = interaction[Constants.Interactions.DIALOGUE_NAME] as? String

        let allParameters = interaction[Constants.Interactions.PARAMETERS] as? [[String: Any]] ?? [[String: Any]]()
        if locale != nil && locale!.trimmingCharacters(in: CharacterSet.whitespaces) != "" {
            self._parameters = getParametersOfLocale(allParameters, locale: locale!)

        } else {
            if let defaultLocale = self._defaultLocale {
                self._parameters = getParametersOfLocale(allParameters, locale: defaultLocale)
            }
        }
        
        self._context = context
        self._locale = locale
    }

    /**
    Returns the interaction id.
    &lt;pre&gt:
    // Swift:
    let interactionId: String? = context.getInteractionId()

    // Objective-C:
    NSString* interactionId = [context getInteractionId];
    &lt;/pre&gt:

    - returns:     The interaction id
    */
    @objc public func getInteractionId() -> String? {
        return self._id
    }
    
    /**
    Returns the interaction name.
    &lt;pre&gt:
    // Swift:
    let interactionName: String? = context.getInteractionName()

    // Objective-C:
    NSString* interactionName = [context getInteractionName];
    &lt;/pre&gt:

    - returns:     The interaction id
    */
    @objc public func getInteractionName() -> String? {
        return self._name
    }
    
    /**
     Returns the plugin type.
     &lt;pre&gt:
     // Swift:
     let pluginType: String? = context.getPluginType()

     // Objective-C:
     NSString* pluginType = [context getPluginType];
     &lt;/pre&gt:

     - returns:     The interaction id
     */
     @objc public func getPluginType() -> String? {
         return self._pluginType
     }
    
    /**
    Returns the id of the Dialogue, only available for interactions.
     It will return nil for all other types
    &lt;pre&gt:
    // Swift:
    let dialogueId: String? = context.getDialogueId()

    // Objective-C:
    NSString* dialogueId = [context getDialogueId];
    &lt;/pre&gt:

    - returns:     The dialogue id
    */
    @objc public func getDialogueId() -> String? {
        return self._dialogueId
    }
    
    /**
    Returns the name of the Dialogue, only available for interactions.
     It will return nil for all other types
    &lt;pre&gt:
    // Swift:
    let dialogueName: String? = context.getDialogueName()

    // Objective-C:
    NSString* dialogueName = [context getDialogueName];
    &lt;/pre&gt:

    - returns:     The dialogue name
    */
    @objc public func getDialogueName() -> String? {
        return self._dialogueName
    }

    /**
    Returns the interaction parameters in a map.
    &lt;pre&gt:
    // Swift:
    let parameters: Dictionary<String, [String]> = context.getParameters()

    // Objective-C:
    NSDictinoary* parameters = [context getParameters];
    &lt;/pre&gt:

    - returns:     The parameters
    */
    @objc public func getParameters() -> [String: [String]] {
        return self._parameters
    }

    /**
    Returns the connection by id.
    &lt;pre&gt:
    // Swift:
    let context: InteractionContext!
    let connection: Connection? = context.getConnection(connectionId)

    // Objective-C:
    InteractionContext* context;
    Connection* connection = [context getConnection:connectionId];
    &lt;/pre&gt:

    - parameter id: The id of the connection
    - returns: The connection.
    */
    @objc public func getConnection(_ id: String) -> Connection? {
        if id == "" {
            return nil
        }
        for connection in BlueConicClient.getInstance(self._context).getConnections() {
            if connection.getId() == id {
                return connection
            }
        }
        return nil
    }

    /**
    Returns a view for the interaction.
    &lt;pre&gt:
    // Swift:
    let view: UIView? = context.getView()

    // Objective-C:
    UIView* view = [context getView];
    &lt;/pre&gt:

    - returns:     The component matching the selector or the position of the interaction or null if no match is found.
    */
    @objc public func getView() -> UIView? {
        return BlueConicClientImpl.getView(self._context, selector: self._positionIdentifier)
    }

    /**
    Returns the 'selector' of the position.
    &lt;pre&gt:
    // Swift:
    let position: String? = context.getPositionIdentifier()

    // Objective-C:
    NSString* position = [context getPositionIdentifier];
    &lt;/pre&gt:

    - returns:     The selector, e.g. "#position_1"
    */
    @objc public func getPositionIdentifier() -> String? {
        return self._positionIdentifier
    }

    /**
     Returns value from parameters.
     - param : key The key for the parameters
     - returns: the value from the parameters
    */
    @objc public func getValueFromParameters(key: String) -> String? {
        if let values = self.getParameters()[key], values.count > 0 && values[0] != "" {
            return values[0]

        }
        return nil
    }

    /**
     Returns the Locale dictionary.
     first, if the locale exists in allParameters return the dictionary based on that locale
     second, if the locale doesn't exists but the default locale does. then return the dictionary based on the default locale
     third, if the locale and the default locale doesn't exists, then return the dictinonary based on the first locale
     fourth, if the are not locales available to use, return an empty dictionary
     - parameter allParameters: all the parameters of an interaction.
     - parameter locale: the used locale for this interaction.
     - returns: Map based on the locale.
    */
    private func getLocaleDictionary(_ allParameters: [[String: Any]], locale: String) -> [[String: Any]] {
        var availableLocales: [String] = []
        for dictionary in allParameters {
            // add all locales to an array
            if let locale = dictionary[Constants.Interactions.LOCALE] as? String {
                availableLocales.append(locale)
            }

            // get the parameters of the locale
            if locale == dictionary[Constants.Interactions.LOCALE] as? String {
                if let parameters = dictionary["parameter"] as? [[String: Any]] {
                    BCLog("Locale used: \(locale)")
                    return parameters
                }
            }
        }

        // if the locale didn't exists as locale, use the default locale instead.
        if let defaultLocale = self._defaultLocale, locale != self._defaultLocale {
            BCLog("The locale '\(locale)' doesn't exists, using default locale '\(defaultLocale)' instead. Based on all locales '\(availableLocales)'", flag: Constants.Flag.WARN)
            return self.getLocaleDictionary(allParameters, locale: defaultLocale)
        } else if !allParameters.isEmpty {
            // if the locale and the default locale didn't exists in the parameters, use the first locale found
            if let firstLocale = allParameters[0][Constants.Interactions.LOCALE] as? String {
                BCLog("The default locale '\(locale)' is not valid, using the first valid locale '\(firstLocale)' instead. Based on all locales: \(availableLocales)", flag: Constants.Flag.WARN)
                return self.getLocaleDictionary(allParameters, locale: firstLocale)
            }
        }

        BCLog("The plugin doesn't contain any locales.")
        return [[String: Any]]()
    }

    private func getParametersOfLocale(_ allParameters: [[String: Any]], locale: String) -> [String: [String]] {
        let parameters = self.getLocaleDictionary(allParameters, locale: locale)
        return BlueConicConfiguration.parametersToDictionary(parameters)
    }

    private func BCLog(_ message: String, flag: String = Constants.Flag.INFO) {
        BlueConicConfiguration.log(message, tag: Constants.Debug.INTERACTION_CONTEXT, flag: flag)
    }

}
