/*
 * $LastChangedBy$
 * $LastChangedDate$
 * $LastChangedRevision$
 * $HeadURL$
 *
 * Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
 */

import Foundation
import WebKit

public class FullscreenLightbox: BlueConicPlugin {
    private var _client: BlueConicClient!
    private var _context: InteractionContext!
    private var _lightbox: BCLightbox!

    public required convenience init(client: BlueConicClient, context: InteractionContext) {
        self.init()
        self._client = client
        self._context = context
    }

    public func onLoad() {
        if self._client.getViewController() == nil {
            return
        }
        let topController = selectTopViewController()
        let iconUrl = getValueFromParameters(key: "iconUrl")

        if let url = getValueFromParameters(key: "url") {
            self._lightbox = BCLightbox.makeURLString(viewController: topController, urlString: url, iconUrl: iconUrl)
        } else if let html = getValueFromParameters(key: "html") {
            self._lightbox = BCLightbox.makeContentFromHtml(viewController: topController, html: html, iconUrl: iconUrl)
        } else {
            let content = getValueFromParameters(key: "content")
            let cssUrl = getValueFromParameters(key: "cssUrl")
            let inlineCss = getValueFromParameters(key: "inlineCss")
            let htmlString = BCLightbox.constructHtml(viewController: topController, content: content, cssUrl: cssUrl, inlineCss: inlineCss)
            self._lightbox = BCLightbox.makeContent(viewController: topController, content: htmlString, baseURL: nil, iconUrl: iconUrl)

        }
        if let margins = getValueFromParameters(key: "margins") {
            self._lightbox.margin = (margins as NSString).floatValue
        }

        if let hideTimer = getValueFromParameters(key: "hideTimer") {
            self._lightbox.hideTimer = Int(hideTimer)
        }

        // If we are loading an url, it will automatically show when the page has finished loading, in all other cases, show when it's done here

        self._lightbox.showIfReady()

    }

    private func selectTopViewController() -> UIViewController {
        var topMostViewController = self._client.getViewController()
        while topMostViewController!.parent != nil {
            topMostViewController = topMostViewController!.parent
        }
        return topMostViewController!
    }

    private func getValueFromParameters(key: String) -> String? {
        let parameters = self._context.getParameters()
        if let values = parameters[key], values.count > 0 && values[0] != "" {
            return values[0]
        }
        return nil
    }
}

// -- BlueConic Lightbox -- Implementation
// -- Represents a pop-up view above the other views --
public class BCLightbox: NSObject, WKNavigationDelegate {
    // public properties
    @objc public var margin: Float = 0
    public var hideTimer: Int?
    // private properties
    private var _viewController: UIViewController!
    private var _backgroundView: UIView = UIView()
    private var _lightboxView: WKWebView = WKWebView()
    private var _closeButton: UIButton = UIButton()
    private var _iconUrl: String?
    private var isLoaded: Bool = false
    private var isReadyForShowing: Bool = false
    private var isShown: Bool = false

    private let lightBoxBackgroundColor: UIColor = UIColor(white: 0, alpha: 0.5)

    @objc public var lightboxView: WKWebView {
        get {
            return self._lightboxView
        }
        set {
            self._lightboxView = newValue
        }
    }

    public func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
        
        if let url = navigationAction.request.url, navigationAction.navigationType == WKNavigationType.linkActivated {
            UIApplication.shared.openURL(url)
            decisionHandler(.cancel)
        } else {
            decisionHandler(.allow)
        }
    }
    
    public func webView(_ webView: WKWebView, didStartProvisionalNavigation navigation: WKNavigation!) {
        if !self.isLoaded {
             self.isLoaded = true
             if self.isReadyForShowing {
                 self.show()
             }
         }
    }

    @objc init(viewController: UIViewController, iconUrl: String?) {
        super.init()
        self._viewController = viewController
        self._iconUrl = iconUrl
    }

    deinit {
        self._lightboxView.removeFromSuperview()
        self._backgroundView.removeFromSuperview()
    }

    @objc public class func constructHtml(viewController: UIViewController, content: String?, cssUrl: String?, inlineCss: String?) -> String {
        var html = ""
        if let content = content {

            let htmlTop = "<!DOCTYPE html><html><head>"
            let htmlMiddle = "</head><body>\n<div class=\"bcFullScreenLightboxWrapper\">\n\t<div class=\"bcFullScreenLightboxMain\">"
            let htmlBottom = "\n</div></div></body></html>"
            let style = self.setHtmlStyle(viewController: viewController, content: content, inlineCss: inlineCss)

            html = htmlTop
            html += style
            if let cssUrl = cssUrl {
                html += "<link rel=\"stylesheet\" type=\"text/css\" href=\"\(cssUrl)\" />"
            }

            html += htmlMiddle

            html += extractContentFromHtml(content: content)

            html += htmlBottom
        }
        return html
    }

    /**
     Extract content from the HTML
     Retrieve only the inside of the body as content.
     - parameter content: The content.
     - returns: the inner HTML of the body tag
     */
    private class func extractContentFromHtml(content: String) -> String {
        var result = content
        if let firstBodyTagRange = result.range(of: "<body>") {
            result = String(result[firstBodyTagRange.upperBound...])
        }

        if let secondBodyTagRange = result.range(of: "</body>") {
            result = String(result[...result.index(before: secondBodyTagRange.lowerBound)])
        }
        return result
    }

    /**
     add default styling for: no margins
     If the content only contains an image:
     center alignment
     scale image according to the width or height
     - parameter viewController: The context.
     - parameter content: The content.
     - parameter inlineCss: Extra inline styling.
     - returns: the style tag
     */
    private class func setHtmlStyle(viewController: UIViewController, content: String, inlineCss: String?) -> String {
        var style = "<style type=\"text/css\">\n"

        // detect if the content only contains an image
        if content.range(of: "<img ") != nil && content.range(of: "<p>") == nil {

            style += "img {max-height: 100%;" +
                "max-width:100%;" +
                "width:100%;" +
                "height:100%;" +
            "object-fit:contain;}\n"
            style += ".bcFullScreenLightboxMain {" +
                "display: -webkit-flex;" +
                "-webkit-align-items: center;" +
                "-webkit-justify-content: center;" +
                "-webkit-flex-direction:column;" +
            "height:100%}\n"
            style += ".bcFullScreenLightboxWrapper { height:100%;}"
            style += "body { height:100vh;}"
        } else {
            style += ".bcFullScreenLightboxMain {padding: 5px;}\n"
        }
        style += "* { margin: 0; padding: 0; }\n"
        style += "</style>"
        if let inlineCss = inlineCss {
            style += "<style type=\"text/css\">\(inlineCss)</style>"
        }
        return style
    }

    @objc public class func makeContentFromHtml(viewController: UIViewController, html: String, iconUrl: String?) -> BCLightbox {
        return makeContent(viewController: viewController, content: html, baseURL: nil, iconUrl: iconUrl)
    }

    @objc public class func makeContent(viewController: UIViewController, content: String, baseURL: URL!, iconUrl: String?) -> BCLightbox {
        let lightbox = BCLightbox(viewController: viewController, iconUrl: iconUrl)
        lightbox.create()
        lightbox.lightboxView.loadHTMLString(content, baseURL: baseURL)
        return lightbox
    }

    @objc public class func makeURLString(viewController: UIViewController, urlString: String, iconUrl: String?) -> BCLightbox {
        return BCLightbox.makeURL(viewController: viewController, url: URL(string: urlString)!, iconUrl: iconUrl)
    }

    @objc public class func makeURL(viewController: UIViewController, url: URL, iconUrl: String?) -> BCLightbox {
        var newUrl = url
        if url.scheme == "" {
            if newUrl.host == nil {
                newUrl = URL(string: "http://\(url.path)")!
            }
        }
        return BCLightbox.makeURLRequest(viewController: viewController, request: URLRequest(url: newUrl), iconUrl: iconUrl)
    }

    @objc public class func makeURLRequest(viewController: UIViewController, request: URLRequest, iconUrl: String?) -> BCLightbox {
        let lightbox = BCLightbox(viewController: viewController, iconUrl: iconUrl)
        lightbox.create()
        lightbox.lightboxView.navigationDelegate = lightbox
        lightbox.lightboxView.load(request)
        return lightbox
    }

    @objc public func showIfReady() {
        self.isReadyForShowing = true
        if self.isLoaded {
            self.show()
        }
    }

    @objc public func show() {
        if let view = self._viewController?.view {
            let verticalMargin = view.bounds.height * (CGFloat(self.margin) / 100)
            let horizontalMargin = view.bounds.width * (CGFloat(self.margin) / 100)
            let topGuide = self._viewController!.topLayoutGuide

            // Settings for Background View
            view.addSubview(self._backgroundView)
            // constraints
            let backgroundDictionary = ["background": self._backgroundView]

            //position constraints
            let horizontalBackgroundConstraint: [NSLayoutConstraint] = NSLayoutConstraint.constraints(withVisualFormat: "H:|-0-[background]-0-|",
                                                                                                      options: NSLayoutConstraint.FormatOptions(rawValue: 0),
                                                                                                      metrics: nil, views: backgroundDictionary)
            let verticalBackgroundConstraint: [NSLayoutConstraint] = NSLayoutConstraint.constraints(withVisualFormat: "V:|-0-[background]-0-|",
                                                                                                    options: NSLayoutConstraint.FormatOptions.alignAllLeading,
                                                                                                    metrics: nil, views: backgroundDictionary)

            self._viewController?.view.addConstraints(horizontalBackgroundConstraint)
            self._viewController?.view.addConstraints(verticalBackgroundConstraint)

            // Settings for Lightbox View
            view.addSubview(self._lightboxView)
            self._lightboxView.addSubview(self._closeButton)

            // constraints
            let lightboxDictionary: [String: Any] = ["lightbox": self._lightboxView, "topGuide": topGuide, "closeButton": self._closeButton]

            //position constraints

            let horizontalConstraint: [NSLayoutConstraint] = NSLayoutConstraint.constraints(withVisualFormat: "H:|-\(horizontalMargin)-[lightbox]-\(horizontalMargin)-|",
                options: NSLayoutConstraint.FormatOptions(rawValue: 0),
                metrics: nil,
                views: lightboxDictionary)

            let verticalConstraint: [NSLayoutConstraint] = NSLayoutConstraint.constraints(withVisualFormat:
                "V:|[topGuide]-\(verticalMargin)-[lightbox]-\(verticalMargin)-|",
                options: NSLayoutConstraint.FormatOptions.alignAllLeading,
                metrics: nil,
                views: lightboxDictionary)

            let horizontalCloseButtonConstraint: [NSLayoutConstraint] = NSLayoutConstraint.constraints(withVisualFormat: "H:[closeButton]-10-|",
                                                                                                       options: NSLayoutConstraint.FormatOptions(rawValue: 0),
                                                                                                       metrics: nil,
                                                                                                       views: lightboxDictionary)

            let verticalCloseButtonConstraint: [NSLayoutConstraint] = NSLayoutConstraint.constraints(withVisualFormat: "V:|-10-[closeButton]",
                                                                                                     options: NSLayoutConstraint.FormatOptions(rawValue: 0),
                                                                                                     metrics: nil,
                                                                                                     views: lightboxDictionary)

            self._viewController?.view.addConstraints(horizontalConstraint )
            self._viewController?.view.addConstraints(verticalConstraint)

            self._viewController?.view.addConstraints(horizontalCloseButtonConstraint)
            self._viewController?.view.addConstraints(verticalCloseButtonConstraint)

            if let hideTimer = self.hideTimer {
                Timer.scheduledTimer(timeInterval: TimeInterval(hideTimer), target: self, selector: #selector(BCLightbox.hide), userInfo: nil, repeats: false)
            }
        }
    }

    @objc public func hide() {
        if self._viewController?.view != nil {

            self._backgroundView.removeFromSuperview()

            self._lightboxView.removeFromSuperview()
            self._closeButton.removeFromSuperview()
        }
    }

    private func create() {
        self._backgroundView = UIView()
        self._lightboxView = WKWebView()
        self._closeButton = UIButton(type: .custom)

        let fmBundle = Bundle(for: BCLightbox.self)
        // if an icon has been specified use that, otherwise use the supplied closing icon
        if let iconUrlString = self._iconUrl {
            if let iconUrl = URL(string: iconUrlString) {
                do {
                    let iconUrlData = try Data(contentsOf: iconUrl)
                    if let image = UIImage(data: iconUrlData) {
                        self._closeButton.setImage(image, for: .normal)
                        self._closeButton.addTarget(self, action: #selector(BCLightbox.hide), for: UIControl.Event.allTouchEvents)
                    }
                } catch _ {
                    self._closeButton.addTarget(self, action: #selector(BCLightbox.hide), for: UIControl.Event.allTouchEvents)
                }
            }
        } else if let imagePath = fmBundle.path(forResource: "close", ofType: "png") {
            if let image = UIImage(contentsOfFile: imagePath) {
                self._closeButton.setImage(image, for: .normal)
                self._closeButton.addTarget(self, action: #selector(BCLightbox.hide), for: UIControl.Event.allTouchEvents)
            }
        }

        if self._viewController != nil {
            let singleFingerTap = UITapGestureRecognizer(target: self, action: #selector(BCLightbox.handleLightBox(_:)))
            self._backgroundView.addGestureRecognizer(singleFingerTap)

            self._lightboxView.translatesAutoresizingMaskIntoConstraints = false
            self._backgroundView.translatesAutoresizingMaskIntoConstraints = false
            self._closeButton.translatesAutoresizingMaskIntoConstraints = false

            _backgroundView.backgroundColor = self.lightBoxBackgroundColor
        }
        self.lightboxView.navigationDelegate = self
    }

    @objc func handleLightBox(_ recognizer: UIGestureRecognizer) {
        hide()
    }
}
