/*
 * $LastChangedBy$
 * $LastChangedDate$
 * $LastChangedRevision$
 * $HeadURL$
 *
 * Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
 */
import Foundation
import UIKit

public class EngagementService: EventSerivceBase {
    
    private let LOG_TAG = "BC_ENGAGMENTSERVICE"
    private var _engagementObject = [String: Any]()
    private var _propertyName: String = ""
    private var _internalPropertyName: String = ""
    
    private var _calculatedIndexValues: Bool = false
    private var _isInterest: Bool = false
    @objc internal var allInterests: [String] = []
    
    private var _changes: [[String: Any]]?
    private var _valuesCount: Int?
    
    private var _days: Int = 0
    private var _useHalfTime: Bool = false
    
    private var _eventManager: BlueConicEventManager!
    
    private var _elementRuleMap = [UIControl: [JSON]]()
    
    /**
     Deinit. Clear the event handlers based on the interaction id of context.
     */
    deinit {
        BlueConicEventFactory.getInstance().clearEventHandlers(self.listenerId)
    }
    
    /**
     Inits everything needed for the engagementservice
     - parameter client: The BlueConicClient instance
     - parameter listenerId: The UUID of the listener
     - parameter propertyName: The profile property value
     - parameter isInterest: Whether the interaction is interested, by default false
     - parameter allInterests: A list of values which are interested, by default []
     - parameter useHalfTime: Whether to use the halftime mechanism, by default false
     */
    public convenience init(client: BlueConicClient, listenerId: String, propertyName: String, isInterest: Bool = false, allInterests: [String] = [], useHalfTime: Bool = false) {
        self.init(client, listenerId: listenerId)
        
        self._propertyName = propertyName
        self._internalPropertyName = "_" + propertyName
        self._useHalfTime = useHalfTime
        
        self._isInterest = isInterest
        self.allInterests = allInterests
        
        let year = self._useHalfTime ? 2017 : 2012
        let month = self._useHalfTime ? 9 : 1
        
        let userCalendar = Calendar.current
        var baseDateComponents = DateComponents()
        // new base is 1 sept 2017
        baseDateComponents.year = year
        baseDateComponents.month = month
        baseDateComponents.day = 1
        (baseDateComponents as NSDateComponents).timeZone = TimeZone(identifier: "UTC")
        
        let base: Date = userCalendar.date(from: baseDateComponents)!
        
        self._days = Int(floor((Date().timeIntervalSince(base) as Double) / (24 * 3600)))
        self._eventManager = BlueConicEventFactory.getInstance()
        //This eventcallback is triggered when a matching event is fired. This is what eventually scores the event.
    }
    
    /**
     Handles an event (eg click event, advanced event), which leads to points for keywords
     - parameter event: Event object
     */
    public override func handleEvent(_ event: Event) {
        self.BCLog("Handling event \(event)")
        
        if let rules = self.getRules(event) {
            for rule in rules {
                if let advancedEvent = event as? AdvancedEvent {
                    self.handleAdvancedEvent(rule, advancedEvent: advancedEvent)
                } else if let updateContentEvent = event as? UpdateContentEvent {
                    self.handleUpdateContentEvent(rule, updateContentEvent: updateContentEvent)
                } else if let clickEvent = event as? ClickEvent {
                    self.handleTouchEvent(rule, clickEvent: clickEvent )
                } else if let formSubmitEvent = event as? FormSubmitEvent {
                    self.handleTouchEvent(rule, clickEvent: formSubmitEvent)
                }
            }
            self.save()
        }
    }
    
    private func applyContentRule (_ rule: JSON) {
        if let selector = ListenerUtil.getSelector(rule[PluginConstants.Listener.TAG_CONTENT_AREA]) {
            addPointsForRule(rule, content: ListenerUtil.getContent(selector, context: nil))
            registerEvent(rule, className: NSStringFromClass(UpdateContentEvent.self))
        } else {
            self.BCLog("Found a rule without a selector")
        }
    }
    
    private func applyTouchRule (_ rule: JSON) {
        if let clickSelector = ListenerUtil.getSelector(rule[PluginConstants.Listener.TAG_CLICKAREA]) {
            if let element = self._client.getView(clickSelector) as? UIControl {
                // map objects with their rule, since we can't set parameters with addTarget
                element.addTarget(self, action: #selector(EngagementService.touchAction(_:)), for: UIControl.Event.touchDown)
                addToRuleMap(element, rule: rule)
                
            } else {
                self.BCLog("Unable to get the view with the selector: \(clickSelector)")
            }
            registerEvent(rule, className: NSStringFromClass(ClickEvent.self))
        } else {
            self.BCLog("Found a click rule without a clickarea selector: \(rule)")
        }
    }
    
    private func applyFormSubmitRule (_ rule: JSON) {
        if let clickSelector = ListenerUtil.getSelector(rule[PluginConstants.Listener.TAG_FORM]) {
            if let element = self._client.getView(clickSelector) as? UIControl {
                // map objects with their rule, since we can't set parameters with addTarget
                element.addTarget(self, action: #selector(EngagementService.touchAction(_:)), for: UIControl.Event.touchDown)
                addToRuleMap(element, rule: rule)
            } else {
                self.BCLog("Unable to get the view with the selector: \(clickSelector)")
            }
            registerEvent(rule, className: NSStringFromClass(FormSubmitEvent.self))
            
        } else {
            self.BCLog("Found a form submit rule without a form selector: \(rule)")
        }
    }
    
    private func applyUrlRule (_ rule: JSON) {
        var content = ""
        
        // We can't reliably check referrers.
        if let urlRule = rule[PluginConstants.Listener.TAG_URL].string,
            PluginConstants.Listener.URL_RULE.contains(urlRule),
            self._client.getScreenName() != "" {
            content += self._client.getScreenName()
        } else {
            self.BCLog("Unable to get the current screenName with the url rule: \(rule)")
        }
        addPointsForRule(rule, content: [content])
    }
    
    /**
     Convenience class to apply rules. This is then transformed into a json array object my mapping each arraymember into a JSON object.
     - parameter rules: An array of Any
     */
    @objc public func applyRules(_ rules: [Any]) {
        applyRules(rules.map({ JSON($0) }))
    }
    
    /*
     * Loops over all available rules and applies them when possible
     * :param: rules  an array of rules in a JSON object
     */
    public func applyRules(_ rules: [JSON]) {
        for rule in rules {
            self.BCLog("Apply rule: \(rule)")
            if let ruleType: String = rule[PluginConstants.Listener.TAG_RULE_TYPE].string {
                switch ruleType {
                case PluginConstants.Listener.RULETYPE_SCORE_CONTENT,
                     PluginConstants.Listener.RULETYPE_INTEREST_CONTENT:
                    applyContentRule(rule)
                case PluginConstants.Listener.RULETYPE_SCORE_CLICK,
                     PluginConstants.Listener.RULETYPE_INTEREST_CLICK:
                    applyTouchRule(rule)
                case PluginConstants.Listener.RULETYPE_SCORE_FORMSUBMIT,
                     PluginConstants.Listener.RULETYPE_INTEREST_FORMSUBMIT:
                    applyFormSubmitRule(rule)
                case PluginConstants.Listener.RULETYPE_INTEREST_URL,
                     PluginConstants.Listener.RULETYPE_SCORE_URL:
                    applyUrlRule(rule)
                case PluginConstants.Listener.RULETYPE_SCORE_SOCIAL_EVENT,
                     PluginConstants.Listener.RULETYPE_SCORE_EVENT,
                     PluginConstants.Listener.RULETYPE_INTEREST_SOCIAL_EVENT,
                     PluginConstants.Listener.RULETYPE_INTEREST_EVENT:
                    registerEvent(rule, className: NSStringFromClass(AdvancedEvent.self))
                default:
                    self.BCLog("Invalid or not supported rule detected", flag: Constants.Flag.WARN)
                }
            }
        }
        
        subscribeListeners()
    }
    
    /**
     The changes object only exists if there have been any changes, convenience function to check this.
     
     - returns:  bool that denotes a change has occured
     */
    @objc public func isChanged() -> Bool {
        guard let changes = self._changes,
            changes.count > 0 else {
                return false
        }
        
        return true
    }
    
    /**
     Save points to BlueConic by setting the changes object to a property.
     */
    @objc public func save() {
        guard let changes = self._changes,
            changes.count > 0  else {
                // we don't have to save anything
                return
        }
        
        let data: String = self._useHalfTime ?
            self.getHalfTimeData(changes) :
            self.getFormattedData(changes)
        
        self.BCLog("Saving: \(self._internalPropertyName) - \(data)")
        self._client.addProfileValue(self._internalPropertyName, value: data)
        
        self._changes = []
    }
    
    /**
     Return the new added points in a format based on the HalfTime mechanism
     - parameter changes: An array of dictionaries containing the changes
     - returns: a stringified JSON structure containing the calculated score of the changes based on the HalfTime mechanism
     */
    private func getHalfTimeData(_ changes: [[String: Any]]) -> String {
        let changeValue: [String: Any] = [
            PluginConstants.Listener.TAG_TIME: ListenerUtil.getCurrentTime(),
            PluginConstants.Listener.TAG_DAYS: self._days,
            PluginConstants.Listener.TAG_DATA: changes
        ]
        
        return ListenerUtil.JSONStringify(changeValue as Any)
    }
    
    /**
     Return the new added points in a format based on the old mechanism
     - parameter changes: An array of dictionaries containing the changes
     - returns: a stringified JSON structure containing the calculated score of the changes based on the old mechanism
     */
    private func getFormattedData(_ changes: [[String: Any]]) -> String  {
        // this is for the engagement score listener: we use the old mechanism
        // ouptput format should look like: {"K":{"p2264":10},"TIME":1521043495061}
        var changeValue = [String : Any]()
        changeValue[PluginConstants.Listener.TAG_TIME] = ListenerUtil.getCurrentTime()
        
        for change: [String: Any] in changes {
            if let name = change[PluginConstants.Listener.SCORE.NAME] as? String,
                let points = change[PluginConstants.Listener.SCORE.POINTS] as? Int {
                
                var data = [String: Int]()
                if let existingData = changeValue[name] as? [String: Int] {
                    data = existingData
                }
                
                let existing = data[PluginConstants.Listener.SCORE.POINTS + String(self._days)] ?? 0
                data[PluginConstants.Listener.SCORE.POINTS + String(self._days)] = existing + points
                changeValue[name] = data
            }
        }
        
        return ListenerUtil.JSONStringify(changeValue as Any)
    }
    
    /**
     Prepares to add points to changes object
     
     - parameter keywords: the keywords that was  matched to
     - parameter score: the number of points to assign
     */
    private func addPoints(_ keywords: [String], score: Int) {
        for keyword in keywords {
            addPoints(keyword, score: score)
        }
    }
    
    /**
     Prepares to add points to changes object
     
     - parameter keyword: the keyword that was  matched to
     - parameter score: the number of points to assign
     */
    private func addPoints(_ keyword: String, score: Int) {
        self._calculatedIndexValues = false
        
        // also add the points to the _changes object
        if self._changes == nil {
            self._changes = [[String: Any]]()
        }
        self._changes?.append([
            PluginConstants.Listener.SCORE.NAME: keyword.lowercased(),
            PluginConstants.Listener.SCORE.POINTS: score
            ])
    }
    
    /**
     Adds point for a rule, depends on whether we're in an interest ranking or not.
     
     - parameter rule: A Json object containing the rule as specified in BlueConic
     - parameter content: The content that was found in the object, aggregated into a String array.
     */
    private func addPointsForRule(_ rule: JSON, content: [String]) {
        if self._isInterest {
            let foundInterests = getInterestsFromContent(rule, content: content)
            addPoints(foundInterests, score: rule[PluginConstants.Listener.TAG_POINTS].intValue)
            // addPointsForInterests(rule, content: content)
        } else if ListenerUtil.contentContainsWord(rule, content: content, interests: ListenerUtil.getWords(rule)) {
            // This is a fixed keyword
            addPoints(PluginConstants.Listener.SCORE_INTEREST, score: rule[PluginConstants.Listener.TAG_POINTS].intValue)
        }
    }
    
    /**
     Adds a certain number of points for the specified interests. Three methods are available for assigning points to arrays, these are:
     * any interests found
     * specified interests
     * interests based on selector
     
     - parameter  rule:  The JSON engagement rule, specifiying how points should be applied to the interests
     - parameter  content:  A string array representing found content for which points can be applied
     */
    //    private func addPointsForInterests(rule: JSON, content: [String]){
    internal func getInterestsFromContent(_ rule: JSON, content: [String]) -> [String] {
        let interestWords = ListenerUtil.getWords(rule)
        if let ruleInterests = rule[PluginConstants.Listener.TAG_INTERESTS].array, ruleInterests.count > 0 {
            if ruleInterests[0].stringValue == PluginConstants.Listener.PRE_ANY {
                //{"interests":["pre_any"]}
                // Any pre-defined interest
                return getEachInterestInContent(rule, content:content, interests:allInterests)
            } else if ListenerUtil.contentContainsWord(rule, content: content, interests: interestWords ) {
                //{"interests":["a","b","c"]}
                // Specific pre-defined interests
                if ruleInterests[0].stringValue == PluginConstants.Listener.PRE_CONTEXT {
                    return content
                }

                return ruleInterests.map {$0.stringValue}
            }
        } else if let selector = rule[PluginConstants.Listener.TAG_INTERESTS][PluginConstants.Listener.TAG_SELECTOR].string {
            //{"interests":{"selector":"#someSelector"}}
            return ListenerUtil.getContent(selector, context: content)
        }
        return []
    }
    
    internal func getEachInterestInContent(_ rule: JSON, content: [String], interests: [String]) -> [String] {
        return interests.filter {ListenerUtil.contentContainsWord(rule, content: content, interests: [$0])}
    }
    
    /**
     Handles an touch event rule as set in blueconic. Similar to handleEventRule, handleContentRule, etc. Unlike other rule handlers this is called when a touch action is detected
     
     - parameter touchRule: A Json object describing the rule as configured in blueconic
     - parameter clickEvent: A Click event
     */
    private func handleTouchEvent(_ touchRule: JSON, clickEvent: ClickEvent) {
        let words: [String] = ListenerUtil.getWords(touchRule)
        let clickArea = ListenerUtil.getSelector(touchRule[PluginConstants.Listener.TAG_CLICKAREA])
        let formArea = ListenerUtil.getSelector(touchRule[PluginConstants.Listener.TAG_FORM])
        if let selector = ListenerUtil.getSelector(touchRule[PluginConstants.Listener.TAG_CONTENT_AREA]),
            (clickArea != nil && clickArea == clickEvent.selector) || (formArea != nil && formArea == clickEvent.selector) {

            if words[0] == PluginConstants.Listener.PRE_ANY &&
                (selector == PluginConstants.Listener.PRE_ANY || selector == PluginConstants.Listener.ANY || selector == PluginConstants.Listener.PRE_CONTEXT) {
                // this code mirrors addPointsForInterests, but doesn't have to check for content in all cases
                if self._isInterest {
                    if let ruleInterests = touchRule[PluginConstants.Listener.TAG_INTERESTS].array, ruleInterests.count > 0 {
                        if ruleInterests[0].stringValue == PluginConstants.Listener.PRE_ANY {
                            let content = ListenerUtil.getContent(selector, context: clickEvent.context)
                            // Any pre-defined interest
                            let interests = getEachInterestInContent(touchRule, content:content, interests: allInterests)
                            addPoints(interests, score: touchRule[PluginConstants.Listener.TAG_POINTS].intValue)
                        } else if ruleInterests[0].stringValue == PluginConstants.Listener.PRE_CONTEXT,
                            let allRuleInterests = clickEvent.context {
                            let content = ListenerUtil.getContent(selector, context: clickEvent.context)
                            let interests = getEachInterestInContent(touchRule, content: content, interests: allRuleInterests)
                            addPoints(interests, score: touchRule[PluginConstants.Listener.TAG_POINTS].intValue)

                        } else {
                            addPoints( ruleInterests.map {$0.stringValue}, score: touchRule[PluginConstants.Listener.TAG_POINTS].intValue)
                        }
                    } else if let ruleInterestSelector = touchRule[PluginConstants.Listener.TAG_INTERESTS][PluginConstants.Listener.TAG_SELECTOR].string {
                        addPoints(ListenerUtil.getContent(ruleInterestSelector, context: clickEvent.context), score: touchRule[PluginConstants.Listener.TAG_POINTS].intValue)
                    }
                    
                } else {
                    addPoints(PluginConstants.Listener.SCORE_INTEREST, score: touchRule[PluginConstants.Listener.TAG_POINTS].intValue)
                }
            } else {
                addPointsForRule(touchRule, content: ListenerUtil.getContent(selector, context: clickEvent.context))
            }
        }
    }
    
    /**
     * Handles the content rule when the content is updated with the update content event
     * - parameter rule: JSON object of the content rule as defined in the listener
     * - parameter updateContentEvent: The update content event
     */
    private func handleUpdateContentEvent(_ rule: JSON, updateContentEvent: UpdateContentEvent) {
        // check if the selector is ok
        if let selector = rule[PluginConstants.Listener.TAG_CONTENT_AREA][PluginConstants.Listener.TAG_SELECTOR].string,
            selector.lowercased().range(of: PluginConstants.Listener.PRE_MOBILE) != nil ||
                selector.lowercased() == updateContentEvent.selector.lowercased() {
            addPointsForRule(rule, content: updateContentEvent.content)
        }
    }
    
    private func handleAdvancedEvent(_ rule: JSON, advancedEvent: AdvancedEvent) {
        let event = rule[PluginConstants.Listener.TAG_EVENT].string
        
        if event == nil || event != advancedEvent.eventName {
            return
        }
        
        /*
         * When no context position has been provided, match with all values within the event context. When no context has
         * been provided match with |. This results in a positive match when "any word", and negative matches for the
         * remaining cases.
         */
        var contextContent = ""
        if let eventContext = advancedEvent.context {
            if let contextPosition = rule[PluginConstants.Listener.TAG_CONTEXT_POSITION].string,
                let contextPositionNr = Int(contextPosition) {
                // When it has been provided, only match with the value at the given context position
                contextContent = eventContext[contextPositionNr - 1]
            } else {
                contextContent = eventContext.joined(separator: "|")
            }
        } else {
            contextContent = "|"
        }
        addPointsForRule(rule, content: [contextContent])
    }
    
    /**
     Respond to touchAction
     - parameter sender: the ui element that triggered the event
     */
    @objc public func touchAction(_ sender: UIControl) {
        self.BCLog("Clicked on \(sender), with rules \(String(describing: self._elementRuleMap[sender]))")
        
        if let touchRules = self._elementRuleMap[sender] {
            for touchRule in touchRules {
                if let selector = touchRule[PluginConstants.Listener.TAG_CLICKAREA][PluginConstants.Listener.TAG_SELECTOR].string {
                    BlueConicEventFactory.getInstance().publish(ClickEvent(selector:selector))
                    BlueConicEventFactory.getInstance().publish(FormSubmitEvent(selector:selector))
                    break
                }
                if let selector = touchRule[PluginConstants.Listener.TAG_FORM][PluginConstants.Listener.TAG_SELECTOR].string {
                    
                    BlueConicEventFactory.getInstance().publish(ClickEvent(selector:selector))
                    BlueConicEventFactory.getInstance().publish(FormSubmitEvent(selector:selector))
                    break
                }
            }
        }
    }
    
    private func addToRuleMap(_ element: UIControl, rule: JSON) {
        if self._elementRuleMap[element] == nil {
            self._elementRuleMap[element] = [rule]
        } else {
            var rules: [JSON] = self._elementRuleMap[element]!
            rules.append(rule)
            self._elementRuleMap[element] = rules
        }
    }
    
    private func BCLog(_ message: String, flag: String = Constants.Flag.INFO) {
        BlueConicConfiguration.log(message, tag: self.LOG_TAG, flag: flag)
    }
}
